# -*- coding: utf-8 -*-

# Copyright © Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see LICENSE for details)

"""Three way merge tests."""

# Local imports
from three_merge import merge


def test_unrelated_additions():
    base = '123456789101112'
    source = '0123456789101112'
    target = '12345678910111213'
    expected = '012345678910111213'
    merged = merge(source, target, base)
    assert merged == expected

    merged = merge(target, source, base)
    assert merged == expected


def test_unrelated_deletions():
    base = '123456789101112'
    source = '1256789101112'
    target = '12345678912'
    expected = '125678912'
    merged = merge(source, target, base)
    assert merged == expected

    merged = merge(target, source, base)
    assert merged == expected


def test_multiple_additions():
    base = '123456789101112'
    source = '1234(56789101112'
    target = '1234567)89101112'
    expected = '1234(567)89101112'
    merged = merge(source, target, base)
    assert merged == expected

    merged = merge(target, source, base)
    assert merged == expected


def test_multiple_deletions():
    base = '123456789101112'
    source = '126789101112'
    target = '123489101112'
    expected = '1289101112'
    merged = merge(source, target, base)
    assert merged == expected

    # breakpoint()
    merged = merge(target, source, base)
    assert merged == expected


def test_addition_deletion():
    base = '123456789101112'
    source = '123456789ab101112'
    target = '123789101112'
    expected = '123789ab101112'
    merged = merge(source, target, base)
    assert merged == expected

    merged = merge(target, source, base)
    assert merged == expected


def test_addition_conflict():
    base = '123456789101112'
    source = '123a456789101112'
    target = '123b456789101112'
    expected = '123<<<<<<< ++ a ======= ++ b >>>>>>>456789101112'
    merged = merge(source, target, base)
    assert merged == expected

    expected = '123<<<<<<< ++ b ======= ++ a >>>>>>>456789101112'
    merged = merge(target, source, base)
    assert merged == expected


def test_addition_deletion_conflict():
    base = '123456789101112'
    source = '123a456789101112'
    target = '12356789101112'
    expected = '123<<<<<<< ++ a ======= -- 4 >>>>>>>56789101112'
    merged = merge(source, target, base)
    assert merged == expected

    expected = '123<<<<<<< ++ a ======= -- 4 >>>>>>>56789101112'
    merged = merge(target, source, base)
    assert merged == expected


def test_deletion_composition():
    base = '123456789101112'
    source = '12346789101112'
    target = '12346789101112'
    expected = '12346789101112'
    merged = merge(source, target, base)
    assert merged == expected


def test_deletion_addition_conflict():
    base = '123456789101112'
    source = '123789101112'
    target = '1234a56789101112'
    expected = '123<<<<<<< ++ a ======= -- 56 >>>>>>>789101112'
    merged = merge(source, target, base)
    assert expected == merged

    merged = merge(target, source, base)
    assert merged == expected


def test_triple_addition():
    base = '123456789101112'
    source = '123\n    456789101112'
    target = '12345678910(11)12'
    expected = '123\n    45678910(11)12'
    merged = merge(source, target, base)
    assert merged == expected

    merged = merge(target, source, base)
    assert merged == expected



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Spyder Project Contributors
#
# Licensed under the terms of the MIT License
# (see LICENSE for details)
# -----------------------------------------------------------------------------
"""Setup script for three_merge."""

# Standard library imports
import ast
import os
import os.path as osp

# Third party imports
from setuptools import find_packages, setup

HERE = osp.dirname(osp.abspath(__file__))


def get_version(module='three_merge'):
    """Get version."""
    with open(os.path.join(HERE, module, '__init__.py'), 'r') as f:
        data = f.read()
    lines = data.split('\n')
    for line in lines:
        if line.startswith('VERSION_INFO'):
            version_tuple = ast.literal_eval(line.split('=')[-1].strip())
            version = '.'.join(map(str, version_tuple))
            break
    return version


def get_description():
    """Get long description."""
    with open(os.path.join(HERE, 'README.md'), 'r') as f:
        data = f.read()
    return data


REQUIREMENTS = [
    'diff-match-patch'
]

EXTRAS_REQUIRE = {
    'test': [
        'pytest',
        'pytest-cov',
        'flaky',
        'pytest-timeout'
    ]
}


setup(
    name='three-merge',
    version=get_version(),
    keywords=['Merge', 'Files', 'Three-way'],
    url='https://github.com/spyder-ide/three-merge',
    license='MIT',
    author='Spyder Project Contributors',
    author_email='spyder.python@gmail.com',
    description='Simple library for merging two strings with respect '
                'to a base one',
    long_description=get_description(),
    long_description_content_type='text/markdown',
    packages=find_packages(exclude=['contrib', 'docs']),
    install_requires=REQUIREMENTS,
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8'
        ],
    extras_require=EXTRAS_REQUIRE
    )
